# SmallOverlap GAP Package
# Copyright (C) 2007-2008 Mark Kambites, University of Manchester
#
# This package contains code for manipulating semigroups and monoids with
# presentations satisfying small overlap conditions of the kind
# introduced by Remmers.  The algorithms used are described in the paper
# M. Kambites, "Small overlap monoids: the word problem", arXiv ref.
#
# This file contains list manipulation routines

########################################################################
# IsFactorAt(u, upos, v, vpos) tests whether the suffix of list v
# beginning at position vpos is a factor of list u beginning at position
# upos. All lists are assumed dense. upos and vpos must be in the ranges
# 1...Length(u) and 1...Length(v) respectively; no checks are made on
# this.

InstallGlobalFunction(IsFactorAt, function(u, upos, v, vpos)
    local i, vlength;
    vlength := Length(v);  # Stored only for efficiency

    if vlength - vpos > Length(u) - upos then
        # the given suffix of v is too long to be a factor of u in
        # the given position
        return false;
    fi;
    for i in [1..(vlength-vpos+1)] do
        if u[i + upos - 1] <> v[i + vpos - 1] then
            return false;
        fi;
    od;
    return true;
end);


########################################################################
# MaxCommonSuffixLength(u,v) returns the length of the maximal common
# suffix of the (dense) lists u and v

InstallGlobalFunction(MaxCommonSuffixLength, function(u, v)
    local i, ulength, vlength;
    i := 0;
    ulength := Length(u); vlength := Length(v);
    while i < ulength and i < vlength and u[ulength-i] = v[vlength-i] do
        i := i + 1;
    od;
    return i;
end);

##########################################################################
# CompareFactors(u, upos, v, vpos, length) checks if the factor of list u
# starting at position upos of length length is equal to the factor of
# list v at position vpos of length length. All entries of u from
# position upos to position upos+length must be bound, and similarly for
# v and vpos.

InstallGlobalFunction(CompareFactors, function(u, upos, v, vpos, length)
   local i;
   if upos+length-1 > Length(u) or vpos+length-1 > Length(v) then
       return false;
   fi;
   for i in [1..length] do
       if u[upos+i-1] <> v[vpos+i-1] then
           return false;
       fi;
   od;
   return true;
end);
