# SmallOverlap GAP Package
# Copyright (C) 2007-2008 Mark Kambites, University of Manchester
#
# This package contains code for manipulating semigroups and monoids with
# presentations satisfying small overlap conditions of the kind
# introduced by Remmers.  The algorithms used are described in the paper
# M. Kambites, "Small overlap monoids: the word problem", arXiv ref.
#
# This file contains exampls to test as much as possible of the code in
# the main word problem algorithm. This is harder than one would think,
# since it is very difficult to construct examples giving rise to some of
# the cases handled by the algorithm; for some cases examples may even not
# exist!


# Make a free monoid on five generators
f := FreeMonoid(["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l"]);
gens := GeneratorsOfMonoid(f);
af := gens[1];
bf := gens[2];
cf := gens[3];
df := gens[4];
ef := gens[5];
ff := gens[6];
gf := gens[7];
hf := gens[8];
iff := gens[9];
jf := gens[10];
kf := gens[11];
lf := gens[12];

#######################################################################
# The following example shows a relation prefix which is not a clean
# overlap prefix which is not clean. For example:
#    RelationPrefix(s, [1,2,3,5],1) = 1
#    CleanOverlapPrefix(s, [1,2,3,5],1) = fail
#
# Also facilitates some quite complex equalities, testing for which
# requires prefix activity checking:
#    C4WordProblem(d*f*a*b*c*d*f, d*f*a*b*c*d*g)
#
# And can be used to test case (3)
#    C4WordProblem(a*b*c*d*f, c*e*g);
#    C4WordProblem(a*b*c*d*f, c*e*f);


# s := f / [[af * bf * cf * df, cf * ef], [df*ff, df*gf]];

########################################################################
# The following is for testing condition (2) in the lemma case analysis
# lemma, eg:
#      C4WordProblem(a*b*c*h*d, a*b*c*d*f);            true
#      C4WordProblem(a*b*c*h*f, a*b*c*d*f);            false
#      C4WordProblem(a*b*c*h*d, a*b*c*h*d);            true
#      C4WordProblem(a*b*c*h*d*f, a*b*c*h*h*d);        true
#
# Also for testing cases (4) and (5) using respectively
#      C4WordProblem(a*b*c*h*d, c*e*f);
#      C4WordProblem(c*e*f, a*b*c*h*d);

# s := f / [[af * bf * cf * df, cf * ef], [df*ff, hf*df]];

########################################################################
#
# This provides a very basic test of condition (6), for example by:
#     C4WordProblem(a*f*d, b*g*d);

# s := f / [[af * ff * hf, bf * gf * hf], [hf * cf, df]];

########################################################################
# The following permits a more complex test of case (6), which also
# involves using the case (2) code to change the prefix being looked for:
#       C4WordProblem(a*f*d*j, b*g*d*j);

# s := f / [[af * ff * hf, bf * gf * hf], [hf * cf, df * ef], [ef*iff,jf]];

# An slightly more complicated presentation for testing case (6), in which
# the max piece suffixes of the first two relation words no longer agree
# (since fh and gh are now pieces).
#       C4WordProblem(a*f*d*j, b*g*d*j);

# s := f / [[af * ff * hf, bf * gf * hf], [hf * cf, df * ef], [ef*iff,jf], [ff*hf*kf,gf*hf*lf]];


########################################################################
# This is a simple example where the GAP's usual (Knuth-Bendix) word
# problem fails, but the new one succeeds.
#   C4WordProblem(a,b);
#   C4WordProblem(a*a*b*c*a*b*c,a*a*b*c*c*b*a);

# s := f / [[af^2 * bf * cf, af * cf * bf * af]];

########################################################################
# The "degenerate" case of the free monoid needs to be tested!

# s := f / [];


########################################################################

a := GeneratorsOfMonoid(s)[1];
b := GeneratorsOfMonoid(s)[2];
c := GeneratorsOfMonoid(s)[3];
d := GeneratorsOfMonoid(s)[4];
e := GeneratorsOfMonoid(s)[5];
f := GeneratorsOfMonoid(s)[6];
g := GeneratorsOfMonoid(s)[7];
h := GeneratorsOfMonoid(s)[8];
i := GeneratorsOfMonoid(s)[9];
j := GeneratorsOfMonoid(s)[10];
k := GeneratorsOfMonoid(s)[11];
l := GeneratorsOfMonoid(s)[12];

I := One(s);
